@@ -20,12 +20,12 @@
 #ifndef DIALOG_H
 #define DIALOG_H
 
-void show_select_game_dialog ();
-void show_game_over_dialog ();
-void show_preferences_dialog ();
-void show_rules_options_dialog ();
-void show_global_stats_dialog ();
-void show_rules_stats_dialog ();
-void show_hint_dialog ();
+void show_select_game_dialog (void);
+void show_game_over_dialog (void);
+void show_preferences_dialog (void);
+void show_rules_options_dialog (void);
+void show_global_stats_dialog (void);
+void show_rules_stats_dialog (void);
+void show_hint_dialog (void);
 
 #endif
