@@ -3,7 +3,7 @@
  * Author: Brent Hendricks
  * Project: GGZ Core Client Lib
  * Date: 9/22/00
- * $Id: netxml.c,v 1.1 2006/10/14 10:27:34 andreasr Exp $
+ * $Id: netxml.c,v 1.2 2006/11/04 22:38:07 andreasr Exp $
  *
  * Code for parsing XML streamed from the server
  *
@@ -1227,7 +1227,8 @@ static void _ggzcore_net_handle_result(GGZNet * net,
 		_ggzcore_room_set_table_leave_status(room, code);
 	else if (strcasecmp(action, "chat") == 0) {
 		if (code != E_OK) {
-		      GGZErrorEventData error = { status:code };
+		      GGZErrorEventData error;
+		      error.status = code;
 
 			switch (code) {
 			case E_NOT_IN_ROOM:
@@ -1272,7 +1273,8 @@ static void _ggzcore_net_handle_result(GGZNet * net,
 	}
 	else if (strcasecmp(action, "admin") == 0) {
 		if (code != E_OK) {
-		    GGZErrorEventData error = { status:code };
+		    GGZErrorEventData error;
+		    error.status = code;
 			snprintf(error.message,
 				 sizeof(error.message),
 				 "Admin action error");
