@@ -3,7 +3,7 @@
  * Author: Brent Hendricks
  * Project: GGZ Core Client Lib
  * Date: 6/5/00
- * $Id: room.c,v 1.1 2006/10/14 10:27:34 andreasr Exp $
+ * $Id: room.c,v 1.2 2006/11/04 22:38:07 andreasr Exp $
  *
  * This fils contains functions for handling rooms
  *
@@ -866,10 +866,10 @@ int _ggzcore_room_admin(GGZRoom *room, GGZAdminType type,
 void _ggzcore_room_add_chat(GGZRoom * room, GGZChatType type,
 			    const char *name, const char *msg)
 {
-      GGZChatEventData data = { type:type,
-	      sender:name,
-	      message:msg
-	};
+      GGZChatEventData data;
+      data.type = type;
+      data.sender = name;
+      data.message = msg;
 
 	ggz_debug(GGZCORE_DBG_ROOM, "Chat (%s) from %s",
 		  ggz_chattype_to_string(type), name);
@@ -890,7 +890,8 @@ void _ggzcore_room_set_table_launch_status(GGZRoom * room, int status)
 	if (status == E_OK) {
 		_ggzcore_room_event(room, GGZ_TABLE_LAUNCHED, NULL);
 	} else {
-	      GGZErrorEventData error = { status:status };
+	      GGZErrorEventData error;
+	      error.status = status;
 
 		switch (status) {
 		case E_BAD_OPTIONS:
@@ -1015,9 +1016,10 @@ void _ggzcore_room_set_table_join_status(GGZRoom * room,
 void _ggzcore_room_set_table_leave(GGZRoom * room,
 				   GGZLeaveType reason, const char *player)
 {
-      GGZTableLeaveEventData event_data = { reason:reason,
-	      player:player
-	};
+      GGZTableLeaveEventData event_data;
+      event_data.reason = reason;
+      event_data.player = player;
+
 	ggz_debug(GGZCORE_DBG_ROOM, "Player left table: %s (%s).",
 		  ggz_leavetype_to_string(reason), player);
 	_ggzcore_server_set_table_leave_status(room->server, E_OK);
